@@ -43,18 +43,9 @@ static FRCommandClass *parent_class = NULL;
 
 
 static gboolean
-fr_command_rar_have_rar (void)
+have_rar (void)
 {
-	char *path;
-	gboolean retval;
-	
-	retval = FALSE;
-	path = g_find_program_in_path ("rar");
-	if (path)
-		retval = TRUE;
-	g_free (path);
-	
-	return retval;
+	return is_program_in_path ("rar");
 }
 
 
@@ -152,14 +143,18 @@ process_line (char     *line,
 		/* read file info. */
 		
 		fields = split_line (line, 6);
-		if ((*fdata->name == 0) || (fields[5][1] == 'D')) {
+
+		if ((fdata->name == NULL)
+		    || (*fdata->name == '\0') 
+		    || (fields[5][1] == 'D') 
+		    || (fields[5][0] == 'd')) {
 			file_data_free (fdata);
 		} else {
-			fdata->size = atol (fields[0]);
+			fdata->size = atoll (fields[0]);
 			fdata->modified = mktime_from_string (fields[3], fields[4]); 
-			g_strfreev (fields);
 			comm->file_list = g_list_prepend (comm->file_list, fdata);
 		}
+		g_strfreev (fields);
 
 		rar_comm->fdata = NULL;
 	}
@@ -177,7 +172,7 @@ add_password_arg (FRCommand	*comm,
 		char *arg, *e_password;
 		
 		e_password = escape_str (password, "\"");
-		g_assert (e_password != NULL);   /* since password is non-NULL */
+		g_assert (e_password != NULL);  /* since password is non-NULL */
 		
 		arg = g_strconcat ("-p\"", e_password, "\"", NULL);
 		
@@ -200,7 +195,7 @@ fr_command_rar_list (FRCommand *comm)
 				      process_line,
 				      comm);
 
-	if (fr_command_rar_have_rar ())
+	if (have_rar ())
 		fr_process_begin_command (comm->process, "rar");
 	else
 		fr_process_begin_command (comm->process, "unrar");
@@ -296,7 +291,7 @@ fr_command_rar_extract (FRCommand  *comm,
 {
 	GList *scan;
 
-	if (fr_command_rar_have_rar ())
+	if (have_rar ())
 		fr_process_begin_command (comm->process, "rar");
 	else
 		fr_process_begin_command (comm->process, "unrar");
@@ -338,7 +333,7 @@ static void
 fr_command_rar_test (FRCommand   *comm,
 		     const char  *password)
 {
-	if (fr_command_rar_have_rar ())
+	if (have_rar ())
 		fr_process_begin_command (comm->process, "rar");
 	else
 		fr_process_begin_command (comm->process, "unrar");
@@ -444,6 +439,9 @@ fr_command_rar_new (FRProcess  *process,
 {
 	FRCommand *comm;
 
+	if ((!is_program_in_path("rar")) && (!is_program_in_path("unrar"))) 
+		return NULL;
+
 	comm = FR_COMMAND (g_object_new (FR_TYPE_COMMAND_RAR, NULL));
 	fr_command_construct (comm, process, filename);
 
